﻿;//////////////////////////////////////////////////////////////////////////////////////////////////
; Help viewer module for cross-platform HTML help files.
;
; This is essentially a visual clone of the help used by PureBasic on OSX and Linux and similar to
; Windows chm help.
;
; The help files themselves are just regular HTML files in a folder. The main folder may have 
; sub-folders and the files might have relative paths for images in other folders.
; There needs to be 2 XML files, contents.xml and index.xml to populate the left panels.  
;
; Public procedures are:
;
;     OpenHelpX(HelpPath.s [, TopicFile.s [, TopicTitle.s]])
;     CloseHelpX()
;
; There is also a public structured data variable gHelpX. This is mostly to give the option of
; controlling the size and position of the help window.
;
; Geoff Looker (aka deeproot) Dec 2018
;//////////////////////////////////////////////////////////////////////////////////////////////////

EnableExplicit

;- DeclareModule
DeclareModule HelpViewX  
  
  Structure HELPVIEWX  
    Split_BarPos.i  
    WinW.i
    WinH.i
    WinX.i
    WinY.i
    HelpPath.s
    TopicFile.s
  EndStructure
  
  Global gHelpX.HELPVIEWX
  
  Declare OpenHelpX(HelpPath.s, TopicFile.s="", TopicTitle.s="")
  Declare CloseHelpX()
  
EndDeclareModule

Module HelpViewX

UsePNGImageDecoder()

#HELP_MINLEFTWIDTH = 200
#HELPX_MenuItem_Search = 59001

;- Structures

Structure HELPVIEWXINTERNAL
  WinHVX.i
  Contain_help.i  
  Button_back.i
  Button_forward.i
  Button_home.i  
  Button_prev.i
  Button_next.i  
  Button_search.i
  List_index.i
  List_search.i
  Panel.i
  Splitter.i
  Str_search.i
  Tree_contents.i
  Web.i
  XML.i
  MenuItem_Search.i
  Split_mousedown.i
  Split_MPos.i
  TopicItem.i
  WinHelpOpen.i
EndStructure


;- Globals

Global gHx.HELPVIEWXINTERNAL
Global Dim gTopicLink.s(0)
Global Dim gIndexLink.s(0)
Global Dim gSearchLink.s(0)

Procedure FillList(*CurrentNode, CurrentSublevel)
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Build the index ListGadget in the left panel
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected *ChildNode
  Protected.s Text$, myIndexItem, myLink
    
  ; Ignore anything except normal nodes
  ;
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
        
    ; Add this node to the tree. Add name and attributes
    ;    
    If CurrentSublevel > 0                                                  ; only inside <index> tag
      
      If ExamineXMLAttributes(*CurrentNode)
        While NextXMLAttribute(*CurrentNode)
          Text$ + XMLAttributeName(*CurrentNode) + "=" + Chr(34) + XMLAttributeValue(*CurrentNode) + Chr(34) + " "
          
          If LCase(XMLAttributeName(*CurrentNode)) = "title"
            myIndexItem = XMLAttributeValue(*CurrentNode)
          ElseIf LCase(XMLAttributeName(*CurrentNode)) = "url"
            myLink = XMLAttributeValue(*CurrentNode)
          EndIf
        Wend
      EndIf
     
      AddGadgetItem(gHx\List_index, -1, myIndexItem)                                 ; add item to ListGadget  
      ReDim gIndexLink(CountGadgetItems(gHx\List_index)-1)
      gIndexLink(CountGadgetItems(gHx\List_index)-1) = myLink                        ; add link to array - at same index

    EndIf
        
    ; Now get the first child node (if any)
    ;    
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    ; Loop through all available child nodes and call this procedure again
    ;
    While *ChildNode <> 0
      FillList(*ChildNode, CurrentSublevel + 1)      
      *ChildNode = NextXMLNode(*ChildNode)
    Wend        
    
  EndIf
 
EndProcedure

Procedure FillTree(*CurrentNode, CurrentSublevel)
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; This procedure fills our TreeGadget, by adding the current node
  ; and then exploring all childnodes by recursively calling itself.
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected *ChildNode
  Protected.s Text$, myTopic, myLink
  ; Ignore anything except normal nodes
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
        
    ; Add this node to the tree. Add name and attributes   
    Text$ = GetXMLNodeName(*CurrentNode) + " (Attributes: "
        
    If CurrentSublevel > 0                                                  ; only inside <contents> tag
      
      If ExamineXMLAttributes(*CurrentNode)
        While NextXMLAttribute(*CurrentNode)
          Text$ + XMLAttributeName(*CurrentNode) + "=" + Chr(34) + XMLAttributeValue(*CurrentNode) + Chr(34) + " "
          
          If LCase(XMLAttributeName(*CurrentNode)) = "title"
            myTopic = XMLAttributeValue(*CurrentNode)
          ElseIf LCase(XMLAttributeName(*CurrentNode)) = "url"
            myLink = XMLAttributeValue(*CurrentNode)
          EndIf
        Wend
      EndIf
      
      Text$ + ")"
      AddGadgetItem(gHx\Tree_contents, -1, myTopic, 0, CurrentSublevel-1)        ; start treegadget at sublevel 0
      ReDim gTopicLink(CountGadgetItems(gHx\Tree_contents)-1)
      gTopicLink(CountGadgetItems(gHx\Tree_contents)-1) = myLink                 ; add link to array - at same index

    EndIf
    
    ; Now get the first child node (if any)
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    ; Loop through all available child nodes and call this procedure again  
    While *ChildNode <> 0
      FillTree(*ChildNode, CurrentSublevel + 1)      
      *ChildNode = NextXMLNode(*ChildNode)
    Wend        
    
  EndIf
  
EndProcedure

Procedure HelpMove()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Get window co-ords on move event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  gHelpX\WinX = WindowX(gHx\WinHVX)
  gHelpX\WinY = WindowY(gHx\WinHVX)  
EndProcedure

Procedure HelpResize()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; resize gadgets
  ;////////////////////////////////////////////////////////////////////////////////////////////////  
  Protected myWinBackColour 
  
  gHelpX\WinW = WindowWidth(gHx\WinHVX)
  gHelpX\WinH = WindowHeight(gHx\WinHVX)
  
  ResizeGadget(gHx\Panel, #PB_Ignore, #PB_Ignore, gHelpX\Split_BarPos, WindowHeight(gHx\WinHVX)-5)
  ResizeGadget(gHx\Tree_contents,  #PB_Ignore, #PB_Ignore, 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight))
  ResizeGadget(gHx\List_index, #PB_Ignore, #PB_Ignore, 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight))  
  ResizeGadget(gHx\Str_search, #PB_Ignore,#PB_Ignore,
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth)-10, #PB_Ignore)
  ResizeGadget(gHx\Button_search, 
               (GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth)-100)/2, 
               #PB_Ignore, #PB_Ignore,#PB_Ignore)
  ResizeGadget(gHx\List_search, #PB_Ignore, #PB_Ignore, 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
               GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight)-65)
  ResizeGadget(gHx\Contain_help, gHelpX\Split_BarPos+12, 0, 
               WindowWidth(gHx\WinHVX)-(gHelpX\Split_BarPos+17), 
               WindowHeight(gHx\WinHVX)-5)
  ResizeGadget(gHx\Web, #PB_Ignore, #PB_Ignore, GadgetWidth(gHx\Contain_help), GadgetHeight(gHx\Contain_help)-38)
  ResizeGadget(gHx\Button_next, GadgetWidth(gHx\Contain_help)-28, 5, #PB_Ignore, #PB_Ignore)
  ResizeGadget(gHx\Button_prev, GadgetWidth(gHx\Contain_help)-64, 5, #PB_Ignore, #PB_Ignore)
  ResizeGadget(gHx\Splitter, gHelpX\Split_BarPos, 0, #PB_Ignore, WindowHeight(gHx\WinHVX))
  
  StartDrawing(WindowOutput(gHx\WinHVX))
  myWinBackColour = Point(0,0)
  StopDrawing()
  CompilerIf #PB_Compiler_OS = #PB_OS_Linux         
    CompilerIf Subsystem("qt")
      myWinBackColour = $CCCCCC                                 ; set grey for qt - back colour always 0?
    CompilerEndIf
  CompilerElseIf #PB_Compiler_OS = #PB_OS_MacOS
    myWinBackColour = $F2F2F2                                   ; very light grey fo Mac
  CompilerEndIf  
  If StartDrawing(CanvasOutput(gHx\Splitter))
    Box(0,0,12,GadgetHeight(gHx\Splitter),myWinBackColour)    
    StopDrawing()    
  EndIf
EndProcedure

Procedure HelpSearch()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; search through each topic files for the given text - show results in list
  ;//////////////////////////////////////////////////////////////////////////////////////////////// 
  Protected i, FileNum
  Protected.s myText, myRec, myFileName
  
  myText = LCase(GetGadgetText(gHx\Str_search))
  If Len(myText) = 0
    ProcedureReturn
  EndIf
  
  ClearGadgetItems(gHx\List_search)
  ReDim gSearchLink(0)
  For i = 0 To ArraySize(gTopicLink())     
    myFileName = gHelpX\HelpPath + gTopicLink(i)
    FileNum = ReadFile(#PB_Any, myFileName)                                                         ; open file
    If FileNum
      While Eof(FileNum) = 0
        If FindString(LCase(ReadString(FileNum)), myText)        
          AddGadgetItem(gHx\List_search, -1, GetGadgetItemText(gHx\Tree_contents,i))                ; add item to ListGadget  
          ReDim gSearchLink(CountGadgetItems(gHx\List_search)-1)
          gSearchLink(CountGadgetItems(gHx\List_search)-1) = gTopicLink(i)                          ; add link to array - at same index
          Break
        EndIf
      Wend
      CloseFile(FileNum)
    EndIf
  Next 
  
EndProcedure

Procedure ContentsTree_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Contents TreeGadget events
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected.s myURL
  If EventType() = #PB_EventType_Change Or 
     EventType() = #PB_EventType_LeftClick Or 
     EventType() = #PB_EventType_LeftDoubleClick
    gHx\TopicItem = GetGadgetState(gHx\Tree_contents)
    If gHx\TopicItem > -1 
      myURL = "file://" + gHelpX\HelpPath + gTopicLink(gHx\TopicItem)
      CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
        myURL = URLEncoder(myURL)     
      CompilerEndIf
      SetGadgetText(gHx\Web, myURL)
    EndIf
  EndIf
EndProcedure

Procedure IndexList_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Index ListGadget events
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected.s myURL
  If GetGadgetState(gHx\List_index) > -1    
    myURL = "file://" + gHelpX\HelpPath + gIndexLink(GetGadgetState(gHx\List_index))
    CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
      myURL = URLEncoder(myURL)  
    CompilerEndIf
  SetGadgetText(gHx\Web, myURL)  
  EndIf 
EndProcedure

Procedure SearchList_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Search ListGadget events
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected.s myURL
  If GetGadgetState(gHx\List_search) > -1    
    myURL = "file://" + gHelpX\HelpPath + gSearchLink(GetGadgetState(gHx\List_search))
    CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
      myURL = URLEncoder(myURL)  
    CompilerEndIf
  SetGadgetText(gHx\Web, myURL)
  EndIf 
EndProcedure

Procedure SearchString_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Search StringGadget events
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  If EventType() = #PB_EventType_Focus    
    AddKeyboardShortcut(gHx\WinHVX, #PB_Shortcut_Return,    #HELPX_MenuItem_Search)
  ElseIf EventType() = #PB_EventType_LostFocus
    RemoveKeyboardShortcut(gHx\WinHVX, #PB_Shortcut_Return)
  EndIf 
EndProcedure

Procedure BackButton_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Back button event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  SetGadgetState(gHx\Web, #PB_Web_Back)    
EndProcedure

Procedure ForwardButton_Handler()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Forward button event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  SetGadgetState(gHx\Web, #PB_Web_Forward)    
EndProcedure

Procedure HomeButton_Handler()  
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Home button event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected.s myURL
  myURL = "file://" + gHelpX\HelpPath + gTopicLink(0)
  CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
    myURL = URLEncoder(myURL)  
  CompilerEndIf
  SetGadgetText(gHx\Web, myURL)  
EndProcedure

Procedure NextTopic()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Next button event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected i
  Protected.s myURL  
  If GetGadgetState(gHx\Panel) = 0
    If gHx\TopicItem < (CountGadgetItems(gHx\Tree_contents) - 1)
      gHx\TopicItem = gHx\TopicItem + 1
      myURL = "file://" + gHelpX\HelpPath + gTopicLink(gHx\TopicItem)
      CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
        myURL = URLEncoder(myURL)  
      CompilerEndIf
      SetGadgetText(gHx\Web, myURL)
    EndIf
  Else    
    myURL = GetGadgetText(gHx\Web)
    CompilerIf #PB_Compiler_OS = #PB_OS_Windows
      myURL = Mid(myURL,Len(gHelpX\HelpPath)+1)
      myURL = ReplaceString(myURL, "\", "/")            
    CompilerElse
      myURL = URLDecoder(myURL)
      myURL = Mid(myURL,Len("file://"+gHelpX\HelpPath)+1)
    CompilerEndIf
    If FindString(myURL,"#")
      myURL = Left(myURL, FindString(myUrl,"#")-1)
    EndIf   
    For i = 0 To ArraySize(gTopicLink())      
      If myURL = gTopicLink(i)
        If i < ArraySize(gTopicLink())
          myURL = "file://" + gHelpX\HelpPath + gTopicLink(i+1)
          CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
            myURL = URLEncoder(myURL)  
          CompilerEndIf
          SetGadgetText(gHx\Web, myURL)                                            ; show next topic          
        EndIf
        Break
      EndIf      
    Next       
  EndIf    
EndProcedure

Procedure PreviousTopic()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Previous button event
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected i
  Protected.s myURL  
  If GetGadgetState(gHx\Panel) = 0                                                 ; on topic contents? 
    If gHx\TopicItem > 0
      gHx\TopicItem = gHx\TopicItem - 1
      myURL = "file://" + gHelpX\HelpPath + gTopicLink(gHx\TopicItem)
      CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
        myURL = URLEncoder(myURL)  
      CompilerEndIf
      SetGadgetText(gHx\Web, myURL)      
    EndIf
  Else                                                                             ; on index?
    myURL = GetGadgetText(gHx\Web)     
    CompilerIf #PB_Compiler_OS = #PB_OS_Windows
      myURL = Mid(myURL,Len(gHelpX\HelpPath)+1)
      myURL = ReplaceString(myURL, "\", "/")            
    CompilerElse
      myURL = URLDecoder(myURL)
      myURL = Mid(myURL,Len("file://"+gHelpX\HelpPath)+1)
    CompilerEndIf     
    If FindString(myURL,"#")
      myURL = Left(myURL, FindString(myUrl,"#")-1)
    EndIf
    For i = 0 To ArraySize(gTopicLink())      
      If myURL = gTopicLink(i)
        If i > 0
          myURL = "file://" + gHelpX\HelpPath + gTopicLink(i-1)
          CompilerIf #PB_Compiler_OS <> #PB_OS_Windows     
            myURL = URLEncoder(myURL)  
          CompilerEndIf
          SetGadgetText(gHx\Web, myURL)                                           ; show previous topic          
        EndIf
        Break
      EndIf      
    Next    
  EndIf    
EndProcedure

Procedure Splitter_Handler()  
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Splitter events handler
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Select EventType()
    Case #PB_EventType_MouseEnter
      SetGadgetAttribute(gHx\Splitter,#PB_Canvas_Cursor,#PB_Cursor_LeftRight)
      
    Case #PB_EventType_MouseLeave
      SetGadgetAttribute(gHx\Splitter,#PB_Canvas_Cursor,#PB_Cursor_Default)
      
    Case #PB_EventType_LeftButtonDown 
      gHx\Split_mousedown=1
      gHx\Split_MPos = GadgetWidth(gHx\Splitter) - GetGadgetAttribute(gHx\Splitter, #PB_Canvas_MouseX)
            
    Case #PB_EventType_LeftButtonUp
      gHx\Split_mousedown=0 
      
    Case #PB_EventType_MouseMove 
      If gHx\Split_mousedown        
        gHelpX\Split_BarPos = GadgetX(gHx\Splitter) + (GetGadgetAttribute(gHx\Splitter, #PB_Canvas_MouseX) - gHx\Split_MPos)
        If gHelpX\Split_BarPos < #HELP_MINLEFTWIDTH
          gHelpX\Split_BarPos = #HELP_MINLEFTWIDTH
        EndIf    
        HelpResize()
      EndIf
      
  EndSelect
  
EndProcedure

Procedure WinOpen_viewer()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Open the Help Viewer window
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  gHx\WinHVX = OpenWindow(#PB_Any, gHelpX\WinX, gHelpX\WinY, gHelpX\WinW, gHelpX\WinH, "Help", 
                          #PB_Window_SystemMenu|#PB_Window_SizeGadget|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget)
  CompilerIf #PB_Compiler_OS = #PB_OS_Linux
    Delay(500)
  CompilerEndIf  
  
  If gHx\WinHVX 
    gHx\WinHelpOpen = #True
    WindowBounds(gHx\WinHVX, 400, 200, #PB_Ignore, #PB_Ignore)
    CompilerIf #PB_Compiler_OS = #PB_OS_MacOS
      SetWindowColor(gHx\WinHVX,$F2F2F2)
    CompilerEndIf
    gHx\Splitter = CanvasGadget(#PB_Any, gHelpX\Split_BarPos, 0, 12, WindowHeight(gHx\WinHVX))
    gHx\Panel = PanelGadget(#PB_Any, 3, 5, gHelpX\Split_BarPos, WindowHeight(gHx\WinHVX)-5)
    AddGadgetItem(gHx\Panel, -1, "Contents")
    gHx\Tree_contents = TreeGadget(#PB_Any, 0, 0, 
                                   GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
                                   GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight), 
                                   #PB_Tree_AlwaysShowSelection|#PB_Tree_NoLines)
    
    AddGadgetItem(gHx\Panel, -1,"Index")
    gHx\List_index = ListViewGadget(#PB_Any, 0, 0, 
                                    GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
                                    GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight))
    
    AddGadgetItem(gHx\Panel, -1,"Search")    
    CompilerIf #PB_Compiler_OS = #PB_OS_Linux
      Delay(500)
    CompilerEndIf
    
    gHx\Str_search = StringGadget(#PB_Any, 5,5,GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth)-10,24,"")
    gHx\Button_search = ButtonGadget(#PB_Any, (GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth)-100)/2, 35, 100,25,"Search")
    gHx\List_search = ListViewGadget(#PB_Any, 0, 65, 
                                     GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemWidth), 
                                     GetGadgetAttribute(gHx\Panel,#PB_Panel_ItemHeight)-65)
    CloseGadgetList()                                                                   
    
    gHx\Contain_help = ContainerGadget(#PB_Any,gHelpX\Split_BarPos+12, 0,WindowWidth(gHx\WinHVX)-(gHelpX\Split_BarPos+17), WindowHeight(gHx\WinHVX)-5) 
    gHx\Button_back = ButtonImageGadget(#PB_Any, 0, 5, 28, 28,ImageID(CatchImage(#PB_Any,?HelpBtnBack)))
    gHx\Button_forward = ButtonImageGadget(#PB_Any, 38, 5, 28, 28,ImageID(CatchImage(#PB_Any,?HelpBtnForward)))
    gHx\Button_home = ButtonImageGadget(#PB_Any, 76, 5, 28, 28,ImageID(CatchImage(#PB_Any,?HelpBtnHome)))
    gHx\Button_next = ButtonImageGadget(#PB_Any, GadgetWidth(gHx\Contain_help)-28, 5, 28, 28,ImageID(CatchImage(#PB_Any,?HelpBtnNext)))
    gHx\Button_prev = ButtonImageGadget(#PB_Any, GadgetWidth(gHx\Contain_help)-64, 5, 28, 28,ImageID(CatchImage(#PB_Any,?HelpBtnPrevious)))
    gHx\Web = WebGadget(#PB_Any, 0, 38, GadgetWidth(gHx\Contain_help), GadgetHeight(gHx\Contain_help)-45, "")
    
    CloseGadgetList()
    
    GadgetToolTip(gHx\Button_back, "back")
    GadgetToolTip(gHx\Button_forward, "forward")
    GadgetToolTip(gHx\Button_home, "home")
    GadgetToolTip(gHx\Button_next, "next topic")
    GadgetToolTip(gHx\Button_prev, "previous topic")
    
    ; Bind events
    BindGadgetEvent(gHx\Splitter,@Splitter_Handler())
    BindGadgetEvent(gHx\Tree_contents,@ContentsTree_Handler())      
    BindEvent(#PB_Event_Gadget, @IndexList_Handler(), gHx\WinHVX, gHx\List_index, #PB_All)
    BindEvent(#PB_Event_Gadget, @SearchList_Handler(), gHx\WinHVX, gHx\List_search, #PB_All)    
    BindGadgetEvent(gHx\Button_back,@BackButton_Handler()) 
    BindGadgetEvent(gHx\Button_forward,@ForwardButton_Handler())
    BindGadgetEvent(gHx\Button_home,@HomeButton_Handler())
    BindGadgetEvent(gHx\Button_prev,@PreviousTopic())
    BindGadgetEvent(gHx\Button_next,@NextTopic())
    BindEvent(#PB_Event_Gadget, @SearchString_Handler(), gHx\WinHVX,gHx\Str_search, #PB_All)
    ;         BindGadgetEvent(gHx\Button_search,@HelpSearch())
    BindEvent(#PB_Event_Gadget, @HelpSearch(), gHx\WinHVX, gHx\Button_search, #PB_All)
    BindEvent(#PB_Event_Menu, @HelpSearch(), gHx\WinHVX, #HELPX_MenuItem_Search)
    BindEvent(#PB_Event_SizeWindow, @HelpResize(), gHx\WinHVX)
    BindEvent(#PB_Event_MoveWindow, @HelpMove(), gHx\WinHVX)
    BindEvent(#PB_Event_CloseWindow, @CloseHelpX(), gHx\WinHVX)       
    
    HelpResize()
  Else
    ProcedureReturn #False
  EndIf
  ProcedureReturn #True     
  
EndProcedure

Procedure XML_ReadFiles()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Read the two XML files and load the Contents and Index in the left panel
  ;////////////////////////////////////////////////////////////////////////////////////////////////  
  Protected.s FileName$, Message$
  Protected *MainNode
  
  FileName$ =  gHelpX\HelpPath + "contents.xml"  
  If FileName$ <> ""    
    gHx\XML = LoadXML(#PB_Any, FileName$)
    If gHx\XML  
      ; Display an error message if there was a markup error
      If XMLStatus(gHx\XML) <> #PB_XML_Success
        Message$ = "Error in the XML file: " + FileName$ + Chr(13)
        Message$ + "Message: " + XMLError(gHx\XML) + Chr(13)
        Message$ + "Line: " + Str(XMLErrorLine(gHx\XML)) + "   Character: " + Str(XMLErrorPosition(gHx\XML))
        MessageRequester("Error", Message$)
      EndIf
      ;      
      ; Get the main XML node, and call the FillTree() procedure with it
      *MainNode = MainXMLNode(gHx\XML)      
      If *MainNode
        FillTree(*MainNode, 0)
      EndIf
                      
      FreeXML(gHx\XML)
      ; Now do the same for the index file
      gHx\XML = LoadXML(#PB_Any, gHelpX\HelpPath + "index.xml") 
      If gHx\XML
        ; Display an error message if there was a markup error
        If XMLStatus(gHx\XML) <> #PB_XML_Success
          Message$ = "Error in the XML file:" + Chr(13)
          Message$ + "Message: " + XMLError(gHx\XML) + Chr(13)
          Message$ + "Line: " + Str(XMLErrorLine(gHx\XML)) + "   Character: " + Str(XMLErrorPosition(gHx\XML))
          MessageRequester("Error", Message$)
        EndIf
        ;
        ; Get the main XML node, and call the FillTree() procedure with it
        *MainNode = MainXMLNode(gHx\XML)      
        If *MainNode
          FillList(*MainNode, 0)
        EndIf  
        FreeXML(gHx\XML)        
      EndIf
      
    Else
      MessageRequester("Error", "The file cannot be opened.", #PB_MessageRequester_Error)
    EndIf
    
  EndIf    
  
EndProcedure

Procedure CloseHelpX()
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Close help window
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  If IsWindow(gHx\WinHVX)
    ; UnBind events
    UnbindGadgetEvent(gHx\Splitter,@Splitter_Handler())
    UnbindGadgetEvent(gHx\Tree_contents,@ContentsTree_Handler())    
    UnbindEvent(#PB_Event_Gadget, @IndexList_Handler(), gHx\WinHVX, gHx\List_index, #PB_All)
    UnbindEvent(#PB_Event_Gadget, @SearchList_Handler(), gHx\WinHVX, gHx\List_search, #PB_All)
    UnbindGadgetEvent(gHx\Button_back,@BackButton_Handler()) 
    UnbindGadgetEvent(gHx\Button_forward,@ForwardButton_Handler())
    UnbindGadgetEvent(gHx\Button_home,@HomeButton_Handler())
    UnbindGadgetEvent(gHx\Button_prev,@PreviousTopic())
    UnbindGadgetEvent(gHx\Button_next,@NextTopic())
    UnbindEvent(#PB_Event_Gadget, @SearchString_Handler(), gHx\WinHVX,gHx\Str_search, #PB_All)    
    UnbindEvent(#PB_Event_Gadget, @HelpSearch(), gHx\WinHVX, gHx\Button_search, #PB_All)
    UnbindEvent(#PB_Event_Menu, @HelpSearch(), gHx\WinHVX, #HELPX_MenuItem_Search)
    UnbindEvent(#PB_Event_SizeWindow, @HelpResize(), gHx\WinHVX)
    UnbindEvent(#PB_Event_MoveWindow, @HelpMove(), gHx\WinHVX)
;     UnbindEvent(#PB_Event_CloseWindow, @CloseHelpX(), gHx\WinHVX)    ; can't unbind in this handler
    
    CloseWindow(gHx\WinHVX)
    ReDim gTopicLink(0)
    ReDim gIndexLink(0)
    ReDim gSearchLink(0)
    gHx\WinHelpOpen = #False
  EndIf
EndProcedure

Procedure OpenHelpX(HelpPath.s, TopicFile.s="", TopicTitle.s="")
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  ; Show Help prodedure - open window, load contents and index, show first help page
  ;////////////////////////////////////////////////////////////////////////////////////////////////
  Protected i
  Protected.s myURL
  
  If FileSize(HelpPath) <> -2 
    MessageRequester("Help Error", "Cannot find the Help files path expected at: " + HelpPath, #PB_MessageRequester_Error)
    ProcedureReturn #False
  EndIf
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows    
    gHelpX\HelpPath = ReplaceString(HelpPath, "\", "/")      
  CompilerElse    
    gHelpX\HelpPath = HelpPath    
  CompilerEndIf
  ; if not already open then open the window and initialise the contents and index from the xml data files
  If Not gHx\WinHelpOpen
    If gHelpX\Split_BarPos < #HELP_MINLEFTWIDTH
      gHelpX\Split_BarPos = 240
      CompilerIf #PB_Compiler_OS = #PB_OS_Linux         
        CompilerIf Not Subsystem("gtk2") And Not Subsystem("qt")
          gHelpX\Split_BarPos = 280                                 ; a little more for gtk3
        CompilerEndIf        
      CompilerEndIf      
    EndIf
    If gHelpX\WinW < 400
      gHelpX\WinW = 720
    EndIf
    If gHelpX\WinH < 200
      gHelpX\WinH = 500
    EndIf
    ExamineDesktops()
    If gHelpX\WinX = 0
      gHelpX\WinX = (DesktopWidth(0)-gHelpX\WinW)/2
    EndIf
    If gHelpX\WinY = 0
      gHelpX\WinY = (DesktopHeight(0)-gHelpX\WinH)/2
    EndIf
        
    If WinOpen_viewer()
      XML_ReadFiles()      
      ; Expand some nodes for a nicer view
      For i = 0 To CountGadgetItems(gHx\Tree_contents) - 1
        If GetGadgetItemAttribute(gHx\Tree_contents, i, #PB_Tree_SubLevel) = 0
          SetGadgetItemState(gHx\Tree_contents, i, #PB_Tree_Expanded)                
        EndIf              
      Next i        
    EndIf
  Else
    If GetWindowState(gHx\WinHVX) = #PB_Window_Minimize
      SetWindowState(gHx\WinHVX, #PB_Window_Normal)
    EndIf
  EndIf 
  
  ; show the topic either by the topic filename or the title
  If Len(TopicFile) > 0
    gHelpX\TopicFile = TopicFile
  ElseIf Len(TopicTitle) > 0     
    gHelpX\TopicFile = ""
    For i = 0 To CountGadgetItems(gHx\Tree_contents) - 1
      If TopicTitle = GetGadgetItemText(gHx\Tree_contents, i)
        gHelpX\TopicFile = gTopicLink(i)
        Break
      EndIf
    Next i
    If Len(gHelpX\TopicFile) = 0
      gHelpX\TopicFile = gTopicLink(0)
    EndIf
  Else    
    gHelpX\TopicFile = gTopicLink(0)
  EndIf  
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows        
    myURL = "file://" + gHelpX\HelpPath + gHelpX\TopicFile    
  CompilerElse     
    myURL = URLEncoder("file://" + gHelpX\HelpPath + gHelpX\TopicFile)    
  CompilerEndIf
  SetGadgetText(gHx\Web, myURL)
  SetActiveWindow(gHx\WinHVX)
  ProcedureReturn #True
  
EndProcedure

;----- DataSection
;
DataSection
;  arrow left
  HelpBtnBack:
  Data.q $0A1A0A0D474E5089,$524448490D000000,$1000000010000000,$FFF31F0000000608,$4D41670400000061,$8A0537C8AF000041,
         $58457419000000E9,$72617774666F5374,$2065626F64410065,$6165526567616D49,$00003C65C9717964,$CB3854414449EB00,
         $8025033FFFFFFC63,$3D280CC830428189,$803149BBC406C991,$32BF9FEFF57F8D05,$CDA0058B08C018A4,$F9FCFEF424D9C517,
         $3F5F8643FDF0F0C3,$18BFC201FAFC337F,$83608EFE9831027E,$6A8164655FF42EE4,$ADC0A7AE2AC6293E,$6E19061FDFE7F0C4,
         $FE205FEFFBFD3039,$03FDFF101FFBF0CD,$5CAF5619BFD10315,$30FFBFD9D002F08B,$D15817F7F5F861FE,$11BFC560DFA2A01F,
         $40FF9FDC9000D434,$33B990BD507F5FB1,$330701889B30BCEE,$BF5E190F97C3C327,$3481AF20D9D5837F,$47C2FEE187FDF9FF,
         $C855763129C88C8C,$0DC706A03942A04D,$CD408D1C904E3B69,$E28CD58907CD40BA,$BBA1E300179C1185,$000000C9D35A7ABC,
         $6042AE444E454900
  Data.b $82
 HelpBtnBackend:
;  arrow right
  HelpBtnForward:
  Data.q $0A1A0A0D474E5089,$524448490D000000,$1000000010000000,$FFF31F0000000608,$4D41670400000061,$8A0537C8AF000041,
         $58457419000000E9,$72617774666F5374,$2065626F64410065,$6165526567616D49,$00003C65C9717964,$CB3854414449EF00,
         $8025033FFFFFFC63,$98080CC430428189,$D91B13926BDBB1E5,$F7E7E6CBF1FAFC06,$50B14118C591575F,$FCFF7F151C13227A,
         $5FEFC30FFBF3F865,$00D40DFCFD7E187F,$968831027E18BFC2,$C337EBF5F8649214,$A0BF6A9D2EB7DBCD,$D84AC93302C3D20F,
         $FEFF01A03FEFE196,$37F8817FBFEFF1C0,$F62617FC407FEFC3,$595A7AAF97CBE187,$02CE65BB4DAF276A,$E8A819AE10A419B3,
         $2117FD046FF15837,$C37F2B906FF7EBF7,$B17B87FEF672005D,$838B9B3067E81B3A,$ABF5E18A5252418B,$7B70BDDC6FB70CD7,
         $0C031018781B3DCE,$DA07EC16802DB50A,$B858E49D59A808FC,$A041F35004F57FD0,$05E784616231756B,$AC6CDD37C0E70800,
         $4E454900000000D0
  Data.b $44,$AE,$42,$60,$82
  HelpBtnForwardend:
;   house
  HelpBtnHome:
  Data.q $0A1A0A0D474E5089,$524448490D000000,$1000000010000000,$FFF31F0000000608,$4D41670400000061,$8A0537C8AF000041,
         $58457419000000E9,$72617774666F5374,$2065626F64410065,$6165526567616D49,$00003C65C9717964,$CB3854414449B802,
         $C71C61524FEB938D,$CBADAEF847F3ABCF,$B646CD30CB25E166,$6566AC60121A3385,$1115E042074D51AE,$8E6CCBCB2980CC61,
         $C0D30BCD228A2386,$36D134D59B065E72,$99339532BC50BB53,$D2AD953876FD22F6,$E7CFBF679BEF8BE6,$D880010E79F7CF39,
         $1E713A49FB45DD2A,$9BB72448A3CE2B19,$451EC2B191EE16DB,$7E5BCD693A9F5AC2,$E82477221CD5EC30,$C4D86460D2403E29,
         $E18ABB5E3AF384C5,$A7C448EE9F0B9DBF,$6409E47815FF9367,$E4C1C96232292A9F,$044ADD26391D1943,$2E9F4622DA31CB3B,
         $7814B7234EA9E22D,$DA3532597F648994,$0146FE0291F043C0,$70C0C86A78E9B5BC,$45580C2AEEE31893,$F2548AFBA9B19770,
         $A796F4993ABF410F,$F82AC7C11F2B7E50,$6A7A60E3A818849B,$088B3E072D6F4992,$49B0F9EB746A7B81,$A0848CC94BE6A853,
         $016DFF2882E9220B,$35E08AA7F4651D82,$5C5C586317CBC09E,$E06766E632F2F2C4,$7767879687431D54,$2C849829E6FDA06E,
         $A9D57C249E3A7F4A,$A3CCE79FD9D37F01,$D2D2C2BF8C4C2CE1,$B419DBECCAFF0312,$E65D12765771B214,$6F673940CC2C680B,
         $EB69F57AE59BA080,$7C6E66D1E879CDC3,$F4F9DFF3B9B73778,$78B86858726A3DF1,$7F70E9D67B996514,$8F61A27DC69D5814,
         $CCABABAB1B2DEB58,$5A82A54A84A952B0,$8D16B329696946AD,$3F139A82DD316146,$B4C99346B9FF8315,$5B5B031C33145142,
         $CACACA86D6D6D15B,$6DF648912A4C414D,$12244AC3060C0EF9,$D9B3602A2A2B8661,$29B3535351DBB760,$05BF850A10E5CB90,
         $3B9D7414272FCF45,$37FE16C406F3B62F,$03468D12623B50C7,$CC5A09D3A701C387,$04A7414763783DAC,$A6D1CADB3A6F1B62,
         $EB2CFE1A09F30F6B,$16F51412C1AA8ABD,$9D0C38708F9AC0A0,$8DDBB705CB97074E,$06682142843ABABA,$44A43B43D31FA854,
         $A0947C59448443BE,$3816DA216DA41E77,$A41672D778A74B2E,$BA406468D1BCE952,$F982040866CD9817,$5F59B5D90B3392DC,
         $F765FC3463A935D3,$4EE9BF76C4835D95,$76EE87AC7D8D1DF3,$45490000000054FD
  Data.b $4E,$44,$AE,$42,$60,$82
  HelpBtnHomeend:  
;    book and arrow left
    HelpBtnPrevious:
  Data.q $0A1A0A0D474E5089,$524448490D000000,$1000000010000000,$FFF31F0000000608,$4D41670400000061,$8A0537C8AF000041,
         $58457419000000E9,$72617774666F5374,$2065626F64410065,$6165526567616D49,$00003C65C9717964,$CB38544144493A02,
         $FE104153684D53A5,$4C55354A6212F25E,$55B43114D6B1262A,$A882B05680951452,$7A1EC1684151E828,$C59E8F0504583C14,
         $1511422C560A1E0A,$131AA96B4208A5E2,$9B18820824B9343D,$EEE36FBBB7BE7F36,$F961748B9B4D4DA6,$567766F99BF76766,
         $3BD7D0CCD8AB56AB,$108F06C739CF1E86,$56B0596E3EAB7D81,$E977D28F89C8F8E6,$0AF5B429463EABFE,$C1F3988F009F7104,
         $31055C892E706E1D,$705C1BF9FBF4C509,$14D6B5510FC2458E,$5C0D780A761D9CDC,$B70D9A066B72AD1F,$F6239CE607AFEDCE,
         $E664773DF7D48B29,$A1B3B87B38FD07D3,$290BABFCB3166F84,$18C66CA932648E57,$ABBEBB77B1BBBBFA,$E3C436BE022B75B5,
         $B39F17FB22095AF1,$2620B3E6CA1E57BD,$E62820D30CA96945,$3224D0E9FAE50A4B,$AF5EBC30E1C20C99,$D7C3AF07F20BAA0A,
         $2DFC19C609D2D1DB,$3F8A728062158541,$320CF13130C5A054,$5D4A9A698050A159,$31A653642E82952A,$40A9871322288B10,
         $CD6664C2910532D3,$50A1E97BFD229847,$EC70E8A3E883FA08,$DF65C872A572E7DA,$0D73E57E2CCFEC82,$B4B206B28C4965D2,
         $7E7D5E33DAF4E9D3,$233D0AC824188179,$531969796C72724E,$212B550C30C02FB7,$910244890E5CB904,$AFEAF94040810448,
         $F17B4BE3223DC820,$B970E411AB6657F8,$8F1E1BB76E8CF55C,$363870E1D75D4D47,$36ED4753BBF9D236,$A2252822F7F1666D,
         $80F462FB3582BA76,$C6E9F85666FE360A,$F763A3721F80D656,$BA9CDDE2FFBDA6CE,$9482FFCDFD03338F,$DA0AA6C50A66332E,
         $4AD3B7B158D55E7D,$BDCAEBB1BF3FF681,$6DE56C94E972E527,$B680E8E4F485B69E,$9577DB4517FCEFD9,$0000000012BE649C,
         $826042AE444E4549
  HelpBtnPreviousend:
 
;   book and arrow right 
  HelpBtnNext:
  Data.q $0A1A0A0D474E5089,$524448490D000000,$1000000010000000,$FFF31F0000000608,$4D41670400000061,$8A0537C8AF000041,
         $58457419000000E9,$72617774666F5374,$2065626F64410065,$6165526567616D49,$00003C65C9717964,$CB38544144495002,
         $7E144113684D53A5,$A62D649A62534D91,$4FE35C5521A4440D,$28A21E2456543D4C,$5EC5145117783D48,$4441127A08888A3C,
         $828788220A1E2C14,$69B6B050A1A85A16,$915A4A9BAD0B6A1A,$333BBA49087A3642,$08398306D4CDF59B,$BEF7DF30C378F81D,
         $E56C36DB61CF7BC7,$82CB2DC44D9F1748,$07F9C3F866279BC1,$280402A0528CE3EF,$26699D5E9E9F88D9,$53F136DA7AFDB8E0,
         $8F7A6E86AD7D226A,$5C063180B6C1CF28,$BAB57035E5F9C788,$A41B8BF4CBEC507A,$02B3247B0C30C041,$B925709C6B8B6303,
         $FC4B21B868E6783F,$AF5D503C78F013F2,$3906201D1DEAEEEE,$4C99490851FFDC3B,$8290ED7428D11206,$364321D5D85E6D6D,
         $55065CB9565E059B,$6201694B13108155,$C80B275CD564809A,$EBAD82850A0E16ED,$0205145079F3E43A,$5B3225BB9DDC0081,
         $14C90CC90CA14B0C,$2C29050B885A14C9,$490F661EA6E75F69,$A95283E7CF81A69A,$8FDAC73CD8662604,$A283AB1A81435583,
         $ABBA19185FD59FAB,$122DBCF9EFAE541C,$A999E4EDDFC20889,$14C04601B22D1985,$6112EF0706581030,$EB7AF5E072E5CA54,
         $EAC656A76DC42E64,$F4EE3C11306EB1B6,$CE5C5B41D05E8277,$861C3852A9BF2F6A,$1DEE2E9B8828B162,$42850ACBC7DA247E,
         $3DB0BD32C7E89ED0,$CCC1BF4D6F97D938,$0CB2C82E898E48AB,$2480BF7EFC1BB76E,$4CC5041B3A74E049,$F81D0D9DEC2FBE20,
         $763127D5CF0E8E31,$209F753EB698B1E9,$EB04E4298A4F0E0C,$98EFC59794B7910F,$E2A2C4F728C374F9,$4C953A7C4A7C6EF2,
         $F0420FA1A1F99D34,$9D070BCECF485D9F,$D4D534646991011D,$B1FC1CF16CD10D41,$24F94A9557FDC2FB,$DEDED25E5F9F651F,
         $CEBAB63A36EFFE49,$DE87728EB83901BF,$454900000000455C
  Data.b $4E,$44,$AE,$42,$60,$82
  HelpBtnNextend:
  
EndDataSection

EndModule


CompilerIf #PB_Compiler_IsMainFile
  ;---- Main
  Define Quit = #False
    
  UseModule HelpViewX
  
  If OpenWindow(0,0,0,500,300,"HelpViewX Test", #PB_Window_SystemMenu|#PB_Window_TitleBar|#PB_Window_ScreenCentered)    
    ButtonGadget(0,50,20,240,40, "Select Help and Open Window")
    ButtonGadget(1,50,80,240,40, "Show Help at references topic file")   
    ButtonGadget(2,50,140,240,40, "Show Help at OpenHelpX topic title")
    ButtonGadget(3,50,200,240,40, "Close Help")  
    TextGadget(4, 300,90, 200, 20,"- if using supplied example")
    TextGadget(5, 300,150, 200, 20,"- if using supplied example")
    DisableGadget(1,1)
    DisableGadget(2,1)
    DisableGadget(3,1)
  EndIf
  
  Repeat
    Select WaitWindowEvent()            
      Case #PB_Event_Gadget     
        Select EventGadget()
          Case 0
            gHelpX\HelpPath = PathRequester("Select Help folder path", GetPathPart(ProgramFilename()))
            OpenHelpX(gHelpX\HelpPath)
            DisableGadget(1,0)
            DisableGadget(2,0)
            DisableGadget(3,0)
          Case 1
            OpenHelpX(gHelpX\HelpPath, "reference/reference.html") 
          Case 2
            OpenHelpX(gHelpX\HelpPath, "", "OpenHelpX") 
          Case 3
            CloseHelpX()
            DisableGadget(1,1)
            DisableGadget(2,1)
            DisableGadget(3,1)
        EndSelect
                
      Case #PB_Event_CloseWindow
        Quit = #True          
        
    EndSelect        
    
  Until Quit
CompilerEndIf
; IDE Options = PureBasic 5.62 (Windows - x64)
; CursorPosition = 705
; FirstLine = 674
; Folding = ----
; EnableXP
; CompileSourceDirectory